package com.example.cubeinspace;


import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;
import android.util.Log;

public class CISGLRenderer implements GLSurfaceView.Renderer {
  
  private static final String TAG = "CISGLRenderer";
  
  /* For basic 2D texture shading, no colour */
  private final String vertexShaderCode = 
      "uniform mat4 uMVPMatrix;  \n" +
          "attribute vec4 vPosition; \n" +
          "attribute vec2 aTexture; \n " +
          "varying vec2 vTexture;   \n" + 
          "void main(){              \n" +
          " gl_Position = uMVPMatrix * vPosition; \n" +
          " vTexture = aTexture; \n" +
          "}                         \n";
  
  /* Uses colour, no texture */
//  private final String vertexShaderCode = 
//      "uniform mat4 uMVPMatrix;  \n" +
//          "attribute vec4 vPosition; \n" +
//          "attribute vec4 aColour; \n" + 
//           "varying vec4 vColour;  \n" +
//          "void main(){              \n" +
//          " vColour = aColour;   \n" +  
//          " gl_Position = uMVPMatrix * vPosition; \n" +
//          "}                         \n";
      
//          " gl_FragColor = vec4 (0.0, 1.0, 1.0, 1.0); \n" +
  
  /* For basic 2D texture shading, no colour*/
    private final String fragmentShaderCode = 
          "precision mediump float;  \n" +
          "uniform sampler2D u_texture; \n" + 
          "varying vec2 vTexture;  \n" +
          "void main(){              \n" +
          " gl_FragColor = texture2D(u_texture, vTexture); \n" +  
          "}                         \n";
    
/* Uses colour, no texture */
//      private final String fragmentShaderCode = 
//          "precision mediump float;  \n" +
//          "varying vec4 vColour;   \n" + 
//          "void main(){              \n" +
//          " gl_FragColor = vColour; \n" + 
//          "}                         \n";

  
  private Square square;
  private Cube cube;
  private int program;
  private int positionHandle;
  private int colourHandle;
  private int textureHandle;
  private Context context;
  
  private int mvpMatrixHandle;
  private float[] uMVPMatrix = new float[16];
  private float[] mMatrix    = new float[16];
  private float[] tMatrix    = new float[16];
  private float[] iMatrix    = new float[16];
  private float[] vMatrix    = new float[16];
  private float[] projMatrix = new float[16];
  
  public float diffX;
  public float diffY;
  
  public CISGLRenderer(Context c) {
    context = c;
  }
  
  public void onSurfaceCreated(GL10 gl, EGLConfig config) {
    GLES20.glClearColor(61f/255, 89f/255, 171f/255, 1.0f);
    GLES20.glEnable(GLES20.GL_DEPTH_TEST);
    GLES20.glDepthFunc(GLES20.GL_LEQUAL);
    square = new Square();
    cube = new Cube(context, R.drawable.yurt);
    
    int vertexShader = loadShader(GLES20.GL_VERTEX_SHADER, vertexShaderCode);
    int fragmentShader = loadShader(GLES20.GL_FRAGMENT_SHADER, 
                                    fragmentShaderCode);
    
    program = GLES20.glCreateProgram();
    GLES20.glAttachShader(program, vertexShader);
    GLES20.glAttachShader(program, fragmentShader);
    GLES20.glBindAttribLocation(program, 0, "vPosition");
    GLES20.glBindAttribLocation(program, 1, "aColour");
    GLES20.glLinkProgram(program);
    
//  Log.v(TAG, "getProgramInfo: " + GLES20.glGetProgramInfoLog(program));
//  Log.v(TAG, "getShaderInfo: " + GLES20.glGetShaderInfoLog(vertexShader));
//  Log.v(TAG, "getShaderInfo: " + GLES20.glGetShaderInfoLog(fragmentShader));
    
    positionHandle = GLES20.glGetAttribLocation(program, "vPosition");
    colourHandle = GLES20.glGetAttribLocation(program, "aColour");
    textureHandle = GLES20.glGetAttribLocation(program, "aTexture");
    mvpMatrixHandle = GLES20.glGetUniformLocation(program, "uMVPMatrix");
    
    Matrix.setIdentityM(mMatrix, 0);
    Matrix.setIdentityM(iMatrix, 0);
    Matrix.setIdentityM(tMatrix, 0);
    Matrix.rotateM(mMatrix, 0, -40, 1, -1, 0);
    Matrix.translateM(iMatrix, 0, 0.3f, 0.3f, 0f);
  }

  public void onDrawFrame(GL10 gl) {
    GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT | GLES20.GL_DEPTH_BUFFER_BIT);
    
//  Matrix.translateM(iMatrix, 0, 0.001f, 0.001f, 0);
    Matrix.translateM(tMatrix, 0, diffX, diffY, 0);
    Matrix.setIdentityM(uMVPMatrix, 0);
    Matrix.multiplyMM(uMVPMatrix, 0, mMatrix, 0, uMVPMatrix, 0);
    Matrix.multiplyMM(uMVPMatrix, 0, tMatrix, 0, uMVPMatrix, 0);
    Matrix.multiplyMM(uMVPMatrix, 0, vMatrix, 0, uMVPMatrix, 0);
    Matrix.multiplyMM(uMVPMatrix, 0, projMatrix, 0, uMVPMatrix, 0);
    Matrix.multiplyMM(uMVPMatrix, 0, iMatrix, 0, uMVPMatrix, 0);
    
    GLES20.glUniformMatrix4fv(mvpMatrixHandle, 1, false, uMVPMatrix, 0);
    GLES20.glUseProgram(program);
    
//  square.draw(positionHandle, textureHandle, context, R.drawable.yurt);
    cube.draw(positionHandle, colourHandle, textureHandle, program);
  }

  public void onSurfaceChanged(GL10 gl, int width, int height) {
    GLES20.glViewport(0, 0, width, height);
    float ratio = (float) width/height;
    Matrix.setLookAtM(vMatrix, 0, 0, 0, -3, 0f, 0f, 0f, 0f, 1.0f, 0.0f);
    Matrix.frustumM(projMatrix, 0, -ratio, ratio, -1, 1, 2, 7);
  }
  
  private int loadShader(int type, String shaderCode) {
    int shader = GLES20.glCreateShader(type);
    GLES20.glShaderSource(shader, shaderCode);
    GLES20.glCompileShader(shader);
    return shader;
  }
}
