package com.example.cubeinspace;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLES20;
import android.opengl.GLUtils;
import android.util.Log;

public class Cube {
  
  private static final String TAG = "Cube";
  private FloatBuffer cubeBuffer;
  private FloatBuffer cubeColourBuffer;
  private FloatBuffer cubeTextureBuffer;
  private ShortBuffer cubeIndexBuffer;
  private int[] textures = new int[1];
  
  private float vertices[] = {
        //Vertices per face
          -0.5f, -0.5f, 0.5f, //v0
          0.5f, -0.5f, 0.5f,  //v1
          -0.5f, 0.5f, 0.5f,  //v2
          0.5f, 0.5f, 0.5f,   //v3
          
          0.5f, -0.5f, 0.5f,  //...
          0.5f, -0.5f, -0.5f,        
          0.5f, 0.5f, 0.5f,
          0.5f, 0.5f, -0.5f,
          
          0.5f, -0.5f, -0.5f,
          -0.5f, -0.5f, -0.5f,        
          0.5f, 0.5f, -0.5f,
          -0.5f, 0.5f, -0.5f,
          
          -0.5f, -0.5f, -0.5f,
          -0.5f, -0.5f, 0.5f,        
          -0.5f, 0.5f, -0.5f,
          -0.5f, 0.5f, 0.5f,
          
          -0.5f, -0.5f, -0.5f,
          0.5f, -0.5f, -0.5f,        
          -0.5f, -0.5f, 0.5f,
          0.5f, -0.5f, 0.5f,
          
          -0.5f, 0.5f, 0.5f,
          0.5f, 0.5f, 0.5f,        
          -0.5f, 0.5f, -0.5f,
          0.5f, 0.5f, -0.5f,
                  };

private float verticesTexture[] = {        
          1.0f, 1.0f,
          0.0f, 1.0f,
          1.0f, 0.0f,
          0.0f, 0.0f,
          
          1.0f, 1.0f,
          0.0f, 1.0f,
          1.0f, 0.0f,
          0.0f, 0.0f,
          
          1.0f, 1.0f,
          0.0f, 1.0f,
          1.0f, 0.0f,
          0.0f, 0.0f,
    
          1.0f, 1.0f,
          0.0f, 1.0f,
          1.0f, 0.0f,
          0.0f, 0.0f,
          
          1.0f, 1.0f,
          0.0f, 1.0f,
          1.0f, 0.0f,
          0.0f, 0.0f,
          
          1.0f, 1.0f,
          0.0f, 1.0f,
          1.0f, 0.0f,
          0.0f, 0.0f,
                    };

/** The initial indices definition */  
private short cubeIndices[] = {
          0,1,3, 0,3,2,       // Front face 
          4,5,7, 4,7,6,       // Right face
          8,9,11, 8,11,10,    //... 
          12,13,15, 12,15,14,   
          16,17,19, 16,19,18,   
          20,21,23, 20,23,22,   
                  };
  
  float verticesColour[] = {
      0.0f, 0.0f, 0.0f, 1.0f,
      0.0f, 0.0f, 1.0f, 1.0f,        
      0.0f, 1.0f, 0.0f, 1.0f,        
      0.0f, 1.0f, 1.0f, 1.0f,        
      1.0f, 0.0f, 0.0f, 1.0f,        
      1.0f, 0.0f, 1.0f, 1.0f,        
      1.0f, 1.0f, 0.0f, 1.0f,        
      1.0f, 1.0f, 1.0f, 1.0f
  };
    
  public Cube(Context context, int textureid) {
    ByteBuffer vbb = ByteBuffer.allocateDirect(vertices.length * 4); 
    vbb.order(ByteOrder.nativeOrder());
    cubeBuffer = vbb.asFloatBuffer();  
    cubeBuffer.put(vertices);   
    cubeBuffer.position(0);   
        
    cubeColourBuffer = ByteBuffer.allocateDirect(
                       verticesColour.length * 4).order(
                       ByteOrder.nativeOrder()).asFloatBuffer();
    cubeColourBuffer.put(verticesColour).position(0);    
    cubeTextureBuffer = ByteBuffer.allocateDirect(
                        verticesTexture.length * 4).order(
                        ByteOrder.nativeOrder()).asFloatBuffer();
    cubeTextureBuffer.put(verticesTexture).position(0); 
    cubeIndexBuffer = ByteBuffer.allocateDirect(
                      cubeIndices.length * 4).order(
                      ByteOrder.nativeOrder()).asShortBuffer();
    cubeIndexBuffer.put(cubeIndices).position(0); 
        
    loadTexture(context, textureid);
  }
  
  public void draw(int positionHandle, int colourHandle, int program) {
    GLES20.glVertexAttribPointer(positionHandle, 3, GLES20.GL_FLOAT, false, 
                                 0, cubeBuffer);
    GLES20.glEnableVertexAttribArray(positionHandle);
    
    GLES20.glVertexAttribPointer(colourHandle, 4, GLES20.GL_FLOAT, false, 
                                 0, cubeColourBuffer);
    GLES20.glEnableVertexAttribArray(colourHandle);
    
    GLES20.glDrawElements(GLES20.GL_TRIANGLES, cubeIndices.length, 
                          GLES20.GL_UNSIGNED_SHORT, cubeIndexBuffer);
  }
  
  public void draw(int positionHandle, int colourHandle, int textureHandle, 
                   int program) {
    GLES20.glVertexAttribPointer(positionHandle, 3, GLES20.GL_FLOAT, false, 
                                 0, cubeBuffer);
    GLES20.glEnableVertexAttribArray(positionHandle);
    
    GLES20.glVertexAttribPointer(colourHandle, 4, GLES20.GL_FLOAT, false, 
                                 0, cubeColourBuffer);
    GLES20.glEnableVertexAttribArray(colourHandle);
    
    GLES20.glVertexAttribPointer(textureHandle, 2, GLES20.GL_FLOAT, false, 
                                 0, cubeTextureBuffer);
    GLES20.glEnableVertexAttribArray(textureHandle);
    GLES20.glUniform1i(textureHandle, 0);
    
    GLES20.glDrawElements(GLES20.GL_TRIANGLES, cubeIndices.length, 
                          GLES20.GL_UNSIGNED_SHORT, cubeIndexBuffer);
  }
  
  public void loadTexture(Context context, int bitmap_id) {
    GLES20.glDeleteTextures(1, textures, 0);
    GLES20.glGenTextures(1, textures, 0);
    GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
    GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textures[0]);
    Bitmap bitmap = BitmapFactory.decodeResource(context.getResources(), 
                                                 bitmap_id);
     
    GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, 
                           GLES20.GL_NEAREST);
    GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, 
                           GLES20.GL_NEAREST);
    GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S, 
                           GLES20.GL_CLAMP_TO_EDGE);
    GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T, 
                           GLES20.GL_CLAMP_TO_EDGE);
      
    GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, bitmap, 0);
    bitmap.recycle();
  }
}
