package com.example.cubeinspace;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLES20;
import android.opengl.GLUtils;
import android.util.Log;

public class Square {
  
  private static final String TAG = "Square";
  private FloatBuffer squareBuffer;
  private FloatBuffer squareColourBuffer;
  private FloatBuffer squareTextureBuffer;
  private int[] textures = new int[1];
  
  float vertices[] = {
        -0.5f, -0.5f, 0.0f, //bottom left
        0.5f, -0.5f, 0.0f,   //bottom right
        -0.5f, 0.5f, 0.0f,   //top left
        0.5f, 0.5f, 0.0f   //top right
                };
  
  float verticesTexture[] = {
      // Texture will be upside-down if you use this set of coordinates
//      0.0f, 0.0f, 0.0f, // bottom left
//      1.0f, 0.0f, 0.0f, // bottom right
//      0.0f, 1.0f, 0.0f, // top left
//      1.0f, 1.0f, 0.0f, // top right
      
      // Texture will be rotated 90 degrees if you use this set of coordinates
//      1.0f, 1.0f, 0.0f, // top right
//      1.0f, 0.0f, 0.0f, // bottom right
//      0.0f, 1.0f, 0.0f, // top left
//      0.0f, 0.0f, 0.0f, // bottom left
      
      // Texture will be correct if you use this set!
      1.0f, 1.0f, 0.0f, // top right
      0.0f, 1.0f, 0.0f, // top left
      1.0f, 0.0f, 0.0f, // bottom right
      0.0f, 0.0f, 0.0f, // bottom left
      
              };
  
  float verticesColour[] = {
      1.0f, 0.0f, 0.0f, 1.0f,
      0.0f, 1.0f, 0.0f, 1.0f,
      0.0f, 0.0f, 1.0f, 1.0f,
      0.0f, 0.0f, 0.0f, 1.0f
  };
  
    
  public Square() {
    ByteBuffer vbb = ByteBuffer.allocateDirect(
                     vertices.length * 4); 
                     vbb.order(ByteOrder.nativeOrder());
    squareBuffer = vbb.asFloatBuffer(); 
    squareBuffer.put(vertices);   
    squareBuffer.position(0); 
        
    squareColourBuffer = ByteBuffer.allocateDirect(
                         verticesColour.length * 4).order(
                         ByteOrder.nativeOrder()).asFloatBuffer();
    squareColourBuffer.put(verticesColour).position(0);    
        
    squareTextureBuffer = ByteBuffer.allocateDirect(
                          verticesTexture.length * 4).order(
                          ByteOrder.nativeOrder()).asFloatBuffer();
    squareTextureBuffer.put(verticesTexture).position(0);    
  }
  
  public void draw(int positionHandle, int textureHandle, Context context, 
                   int bitmap_id) {
    Bitmap bitmap = BitmapFactory.decodeResource(context.getResources(), 
                    bitmap_id);
    GLES20.glDeleteTextures(1, textures, 0);
    GLES20.glGenTextures(1, textures, 0);
    GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
    GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textures[0]);
    // Setting MIN_FILTER required if not using mipmaps
    GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, 
                           GLES20.GL_LINEAR);
    GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, bitmap, 0);
    bitmap.recycle();
    
    GLES20.glVertexAttribPointer(positionHandle, 3, GLES20.GL_FLOAT, false, 
                                 0, squareBuffer);
    GLES20.glEnableVertexAttribArray(positionHandle);
    GLES20.glVertexAttribPointer(textureHandle, 3, GLES20.GL_FLOAT, false, 
                                 0, squareTextureBuffer);
    GLES20.glEnableVertexAttribArray(textureHandle);
    
    GLES20.glDrawArrays(GLES20.GL_TRIANGLE_STRIP, 0, vertices.length/3);
  }
}
